#include "config.h"

#include <stdlib.h>
#include <string.h>

#include <glib.h>

#include "test-stuff.h"

#include "gnc-xml-helper.h"
#include "gnc-engine-util.h"
#include "sixtp.h"
#include "sixtp-parsers.h"
#include "sixtp-utils.h"
#include "sixtp-dom-parsers.h"
#include "sixtp-dom-generators.h"

#include "gnc-commodity.h"
#include "guid.h"

static void
test_dom_tree_to_commodity_ref(void)
{
    int i;
    for(i = 0; i < 20; i++)
    {
        gnc_commodity *test_com1;
        gchar *test_str1;
        gchar *test_str2;
        gnc_commodity *test_com2;
        xmlNodePtr test_node;

        test_str1 = get_random_string();
        test_str2 = get_random_string();
        
        test_com1 = gnc_commodity_new(NULL, test_str1, test_str2, NULL, 0);
        test_node = commodity_ref_to_dom_tree("test-com", test_com1);

        test_com2 = dom_tree_to_commodity_ref_no_engine(test_node);

        if(gnc_commodity_equiv(test_com1, test_com2))
        {
            success(i, "dom_tree_to_commodity_ref_no_engine", "");
        }
        else
        {
            failure(i, "dom_tree_to_commodity_ref_no_engine", "");
        }

        xmlFreeNode(test_node);
        gnc_commodity_destroy(test_com1);
        gnc_commodity_destroy(test_com2);
        g_free(test_str1);
        g_free(test_str2);
    }
}

static void
test_dom_tree_to_text(void)
{
    int i;

    for(i = 0; i < 20; i++)
    {
        gchar *test_string1;
        gchar *test_string2;
        xmlNodePtr test_node;

        test_node = xmlNewNode(NULL, "test-node");
        test_string1 = get_random_string();

        xmlNodeAddContent(test_node, test_string1);

        test_string2 = dom_tree_to_text(test_node->xmlChildrenNode);

        if(safe_strcmp(test_string1, test_string2) == 0)
        {
            success(i, "dom_tree_to_string", "with string %s", test_string1);
        }
        else
        {
            failure(i, "dom_tree_to_string", "with string %s", test_string1);
        }

        xmlFreeNode(test_node);
        g_free(test_string1);
        g_free(test_string2);
    }
}


static void
test_dom_tree_to_timespec(void)
{
    int i;
    for(i = 0; i < 20; i++)
    {
        Timespec *test_spec1;
        Timespec *test_spec2;
        xmlNodePtr test_node;

        test_spec1 = get_random_timespec();

        test_node = timespec_to_dom_tree("test-spec", test_spec1);

        test_spec2 = dom_tree_to_timespec(test_node);

        if(test_spec2 == NULL)
        {
            failure(i, "dom_tree_to_timespec", "NULL return");
            printf("Node looks like:\n");
            xmlElemDump(stdout, NULL, test_node);
            printf("\n");
        }
        
        else if(timespec_cmp(test_spec1, test_spec2) == 0)
        {
            success(i, "dom_tree_to_timespec", "");
        }
        else
        {
            failure(i, "dom_tree_to_timespec", "");
            printf("Node looks like:\n");
            xmlElemDump(stdout, NULL, test_node);
            printf("\n");
            printf("Secs are %lld vs %lld :: ",
                   test_spec1->tv_sec,
                   test_spec2->tv_sec);
            printf("NSecs are %ld vs %ld\n",
                   test_spec1->tv_nsec,
                   test_spec2->tv_nsec);
        }

        g_free(test_spec1);
        g_free(test_spec2);
        xmlFreeNode(test_node);
    }
}


static void
test_dom_tree_to_gnc_numeric(void)
{
    int i;
    for(i = 0; i < 20; i++)
    {
        gnc_numeric test_num1;
        gnc_numeric *test_num2;
        xmlNodePtr test_node;

        test_num1 = get_random_gnc_numeric();
        
        test_node = gnc_numeric_to_dom_tree("test-num", &test_num1);
        test_num2 = dom_tree_to_gnc_numeric(test_node);

        if(!test_num2)
        {
            failure(i, "dom_tree_to_gnc_numeric", "NULL return");
        }
        else if(gnc_numeric_equal(test_num1, *test_num2))
        {
            success(i, "dome_tree_to_gnc_numeric", "");
        }
        else
        {
            failure(i, "dome_tree_to_gnc_numeric", "");
        }

        xmlFreeNode(test_node);
        g_free(test_num2);
    }
}


static void
test_dom_tree_to_guid(void)
{
    int i;
    for(i = 0;i < 20; i++)
    {
        GUID *test_guid1;
        GUID *test_guid2;
        xmlNodePtr test_node;
        
        test_guid1 = get_random_guid();
        
        if (!(test_node = guid_to_dom_tree("test-guid", test_guid1)))
        {
            failure(i, "guid_to_dom_tree", "conversion to dom tree failed");
        }

        test_guid2 = dom_tree_to_guid(test_node);

        if(!guid_equal(test_guid1, test_guid2))
        {
            failure(i, "dom_tree_to_guid", "");
        }
        else
        {
            success(i, "dom_tree_to_guid", "");
        }

        xmlFreeNode(test_node);
        g_free(test_guid1);
        g_free(test_guid2);
    }
}

int
main(int argc, char **argv)
{
    test_dom_tree_to_guid();
    fflush(stdout);
    test_dom_tree_to_commodity_ref();
    fflush(stdout);
    test_dom_tree_to_text();
    fflush(stdout);
    test_dom_tree_to_timespec();
    fflush(stdout);
    test_dom_tree_to_gnc_numeric();
    fflush(stdout);
    exit(get_rv());
}
