#include "config.h"

#include <glib.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "gnc-xml-helper.h"
#include "gnc-xml.h"

#include "sixtp-parsers.h"

#include "test-stuff.h"

#include "Account.h"

static void
set_account_random_string(Account* act,
                          void(*func)(Account *act, const gchar*str))
{
    gchar *tmp_str = get_random_string();
    (func)(act, tmp_str);
    g_free(tmp_str);
}

static Account*
get_random_account(void)
{
    Account *ret;
    int tmp_int;
    
    ret = xaccMallocAccount();

    xaccAccountBeginEdit(ret);

    set_account_random_string(ret, xaccAccountSetName);

    tmp_int = get_random_int_in_range(BANK, CREDITLINE);
    xaccAccountSetType(ret, tmp_int);

    set_account_random_string(ret, xaccAccountSetCode);
    set_account_random_string(ret, xaccAccountSetDescription);

    xaccAccountSetCurrency(ret, get_random_gnc_commodity_ref());
    xaccAccountSetSecurity(ret, get_random_gnc_commodity_ref());
    
    xaccAccountSetSlots_nc(ret, get_random_kvp_frame());
    
    xaccAccountCommitEdit(ret);
    return ret;
}

static void
test_generation(void)
{
    int i;
    for(i = 0; i < 20; i++)
    {
        Account *ran_act;
        xmlNodePtr test_node;
        gchar *filename1;
        gchar *filename2;
        FILE *cmp_file;
        int fd;
        
        ran_act = get_random_account();

        test_node = gnc_account_dom_tree_create(ran_act);
        if(!test_node)
        {
            failure(i, "account_xml",
                    "gnc_account_dom_tree_create returned NULL");
            xaccAccountDestroy(ran_act);
            continue;
        }

        filename1 = g_strdup_printf("test_file_XXXXXX");
        
        fd = mkstemp(filename1);
        
        write_dom_node_to_file(test_node, fd);

        close(fd);

        filename2 = g_strdup_printf("test_file2_XXXXXX");
        fd = mkstemp(filename2);
        cmp_file = fdopen(fd, "w");
        
        sixtp_parse_file(sixtp_dom_parser_new(print_dom_tree, NULL, NULL),
                         filename1, NULL, (gpointer)cmp_file, NULL);
        fclose(cmp_file);
        close(fd);

        if(files_compare(filename1, filename2) == 0)
        {
            success(i, "account_xml", "");
        }
        else
        {
            failure(i, "account_xml", "");
        }

        unlink(filename1);
        unlink(filename2);
        g_free(filename1);
        g_free(filename2);
        xaccAccountBeginEdit(ran_act);
        xaccAccountDestroy(ran_act);
        xmlFreeNode(test_node);
    }
}

int
main(int argc, char **argv)
{
    test_generation();
    exit(get_rv());
}
