#include "config.h"

#include <glib.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "gnc-xml-helper.h"
#include "gnc-xml.h"

#include "sixtp-parsers.h"

#include "test-stuff.h"

#include "Account.h"

static gnc_commodity*
get_random_commodity(void)
{
    gnc_commodity *ret;
    gchar *name;
    gchar *space;
    gchar *mn;
    gchar *xcode;
    int ran_int;

    name = get_random_string();
    space = get_random_string();
    mn = get_random_string();
    xcode = get_random_string();
    ran_int = get_random_int_in_range(1, 100000);

    ret = gnc_commodity_new(name, space, mn, xcode, ran_int);

    g_free(name);
    g_free(space);
    g_free(mn);
    g_free(xcode);

    return ret;
}


static void
test_generation(void)
{
    int i;
    for(i = 0; i < 20; i++)
    {
        gnc_commodity *ran_com;
        xmlNodePtr test_node;
        gchar *filename1;
        gchar *filename2;
        FILE *cmp_file;
        int fd;
        
        ran_com = get_random_commodity();

        test_node = gnc_commodity_dom_tree_create(ran_com);
        if(!test_node)
        {
            failure(i, "commodity_xml",
                    "gnc_commodity_dom_tree_create returned NULL");
            gnc_commodity_destroy(ran_com);
            continue;
        }

        filename1 = g_strdup_printf("test_file_XXXXXX");
        
        fd = mkstemp(filename1);
        
        write_dom_node_to_file(test_node, fd);

        close(fd);

        filename2 = g_strdup_printf("test_file2_XXXXXX");
        fd = mkstemp(filename2);
        cmp_file = fdopen(fd, "w");
        
        sixtp_parse_file(sixtp_dom_parser_new(print_dom_tree, NULL, NULL),
                         filename1, NULL, (gpointer)cmp_file, NULL);
        fclose(cmp_file);
        close(fd);

        if(files_compare(filename1, filename2) == 0)
        {
            success(i, "commodity_xml", "");
        }
        else
        {
            failure(i, "commodity_xml", "");
        }

        unlink(filename1);
        unlink(filename2);
        g_free(filename1);
        g_free(filename2);
        gnc_commodity_destroy(ran_com);
        xmlFreeNode(test_node);
    }
}

int
main(int argc, char **argv)
{
    test_generation();
    exit(get_rv());
}
